''************************************
''*  Stepper Driver v1.01            *
''*  By Alberto Geraci               *
''*  BTX Sistemas                    * 
''************************************

'' PaP Pulses driver controller with cuadratic acceleration/deceleration.
'' Can be tested with any driver with L297 chip or similar. - Sig Positec D900 - NWE SD2202
'' Improved driver with acceleration and speed, controlled each by a potenciometer, using RCTime.
'' Pins 20,22,24 are flags indicating that a motor is turning.

CON
  CeroX = 1         ' Zero X input
  CeroY = 2         '
  CeroZ = 3         '
  DcerX = 11        ' DIR of Zero inputs.
  DcerY = 12        '
  DcerZ = 13        '
  PinX  = 4         ' Output for pulses
  PinY  = 6         '
  PinZ  = 8         '
  K     = 180000    ' Constant arbitrary
    
VAR
  long  Stack[20]             'Stack space for new cogs
  long  a
  long  b
  long  c
  byte  Cog
  byte  Temp
  long  Rs
  
PUB Start(Pin,Dir,Zero,Sp,Rt,Se) : Success 
   Stop
   Success :=  (Cog := cognew(Turn(Pin,Dir,Zero,Sp,Rt,Se), @Stack) + 1)

PUB Stop
{{Stop toggling process, if any.}}

  if Cog
    cogstop(Cog~ - 1)

PUB Turn(Pin,Dir,Zero,Sp,Rt,Se) | pc,R       ' Stepper run rutine
{{
'   Pin= Output pulses pin
'   Dir= CW or CCW turn  (Logical state depends of the driver that you use...lets choice yourself)
'   Sp = Speed of motor (In units).
'   Rs = Ramp Size (quantity of pulses used per ramp.)
'   Rt = Ramp Step Time (Increment of the step time in ramp. 'Units')
'   Se = Steps to turn.
}}
  pc          := 0             ' pc = pulses counter
  Rs := k/Rt
  a  := (Rt*Rt)/k              ' NEW cuadratic ramp aX2 + bX + c
  b  := -2 * Rt
  c  := k + Sp 


  R           := Rs * Rt       ' Initial ramp time for start
  dira[Pin..Pin+1]~~           ' Set I/O Pin to output direction  PULSES OUT, DIR OUT                                         
  dira[19]~~                   ' Set I/O Pin to output direction  ERROR LED (If used)                                        
  dira[CeroX..CeroZ]~          ' Direcciones de los pines par cero maquina en entradas.
  dira[DcerX..DcerZ]~
  dira[Pin + 16]~~             ' Direccion de los pines de motor girando.
'***************************************************************************************
'  Movimiento normal por cantidad de pulsos mandada.
'***************************************************************************************
  Ifnot Zero                   
'***************************************************************************************

       outa[Pin+1] := Dir               ' Set dir pin state
       outa[Pin + 16]~~                 ' Set high TurnPin => Motors turning
       repeat Se                        ' Loop turn the motor "Se" steps....
         pc    := pc + 1
         IF Rs*2 =< Se 
           if pc =< Rs
             R := a*(pc*pc) + b*pc + c
           if pc => Se - Rs
             R := a*(Se-pc)*(Se-pc) + b*(Se-pc) + c
         else
           if pc =< Se/3                
             R := a*(pc*pc) + b*pc + c
           if pc => Se - (Se/3)                                    
             R := a*(Se-pc)*(Se-pc) + b*(Se-pc) + c

         outa[Pin] := 1               ' Set pin 4 high
         waitcnt(R + 1000 + cnt )     ' Wait for pulse high width plus 1000 to adjust the time lost in other processing.)
         outa[Pin] := 0               ' Pin 4 now low
         waitcnt(R + cnt)             ' Wait for pulse low width)......50% duty cycle to motor driver                           
       outa[Pin + 16]~                ' Set low TurnPin => Motors Stoped

'***************************************************************************************
'  Movimiento de cantidad de pulsos ilimitada para cero maquina.
'***************************************************************************************
  else
'***************************************************************************************
       Sp := 50000                    ' Fix speed for zero machine
       c  := k + Sp 
      Temp := Ina[DcerX]
       If Pin == PinX
          If Temp
            outa[Pin+1]~~
          else
            outa[Pin+1]~
       Temp := Ina[DcerY]
       If Pin == PinY
          If Temp
            outa[Pin+1]~~
          else
            outa[Pin+1]~
       Temp := Ina[DcerZ]
       If Pin == PinZ
          If Temp
            outa[Pin+1]~~
          else
            outa[Pin+1]~
           
       outa[Pin + 16]~~                ' Set high TurnPin => Motors turning
       repeat                         ' Loop turn the motor until CeroX,Y,Z sense high level
         Temp := Ina[CeroX]
         If Pin == PinX
          If Temp
            quit
         Temp := Ina[CeroY]
         If Pin == PinY
          If Temp
            quit
         Temp := Ina[CeroZ]
         If Pin == PinZ
          If Temp
            quit

         pc    := pc + 1
         if pc =< Rs                                         ' Acc ramp
          R := a*(pc*pc) + b*pc + c
         outa[Pin] := 1               ' Set pin 4 high
         waitcnt(R + cnt )       ' Wait for pulse high width )
         outa[Pin] := 0               ' Pin 4 now low
         waitcnt(R + cnt)        ' Wait for pulse low width)......50% duty cycle to motor driver                           
       outa[Pin + 16]~                 ' Set low TurnPin => Motors Stoped
                                                     